@@ -93,6 +93,10 @@ extern int dpdk_port_deinit(int port);
 #define SERIALIZATION_BUFSIZ     (8192 * 2)
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 // inner hash table (ja3 -> security state)
 typedef struct ndpi_ja3_info {
   char * ja3;
@@ -166,7 +170,6 @@ enum info_type {
     INFO_TIVOCONNECT,
     INFO_FTP_IMAP_POP_SMTP,
     INFO_NATPMP,
-    INFO_RTP
 };
 
 // flow tracking
@@ -206,6 +209,7 @@ typedef struct ndpi_flow_info {
   ndpi_protocol detected_protocol;
   ndpi_confidence_t confidence;
   u_int16_t num_dissector_calls;
+  u_int16_t dpi_packets;
 
   // Flow data analysis
   pkt_timeval src2dst_last_pkt_time, dst2src_last_pkt_time, flow_last_pkt_time;
@@ -213,30 +217,36 @@ typedef struct ndpi_flow_info {
     *pktlen_c_to_s, *pktlen_s_to_c;
 
   enum info_type info_type;
+
   union {
     char info[256];
+    
     struct {
       unsigned char auth_failed;
       char username[127];
       char password[128];
     } ftp_imap_pop_smtp;
+    
     struct {
       char domain[85];
       char hostname[85];
       char username[86];
     } kerberos;
+    
     struct {
       char ip[16];
       char port[6];
       char hostname[48];
       char fqdn[48];
     } softether;
+    
     struct {
       char identity_uuid[36];
       char machine[48];
       char platform[32];
       char services[48];
     } tivoconnect;
+    
     struct  {
       uint16_t result_code;
       uint16_t internal_port;
@@ -247,12 +257,15 @@ typedef struct ndpi_flow_info {
 
   ndpi_serializer ndpi_flow_serializer;
 
-  char flow_extra_info[16];
   char host_server_name[80]; /* Hostname/SNI */
   char *bittorent_hash;
   char *dhcp_fingerprint;
   char *dhcp_class_ident;
   ndpi_risk risk;
+
+  struct {
+    char currency[16];
+  } mining;
   
   struct {
     u_int16_t ssl_version;
@@ -270,17 +283,19 @@ typedef struct ndpi_flow_info {
       char *esni;
     } encrypted_sni;    
 
+    struct {
+      u_int16_t version;
+    } encrypted_ch;
+
     time_t notBefore, notAfter;
     u_int16_t server_cipher;
     ndpi_cipher_weakness client_unsafe_cipher, server_unsafe_cipher;
+
+    u_int32_t quic_version;
   } ssh_tls;
 
   struct {
-    enum ndpi_rtp_stream_type stream_type;
-  } rtp;
-
-  struct {
-    char url[256], request_content_type[64], content_type[64], user_agent[256], server[128];
+    char url[256], request_content_type[64], content_type[64], user_agent[256], server[128], nat_ip[32], filename[256];
     u_int response_status_code;
   } http;
 
@@ -288,6 +303,12 @@ typedef struct ndpi_flow_info {
     char *username, *password;
   } telnet;
 
+  struct {
+    char geolocation_iata_code[4];
+  } dns;
+
+  ndpi_multimedia_flow_type multimedia_flow_type;
+  
   void *src_id, *dst_id;
 
   struct ndpi_entropy *entropy;
@@ -392,15 +413,14 @@ int ndpi_is_datalink_supported(int datalink_type);
 /* compare two nodes in workflow */
 int ndpi_workflow_node_cmp(const void *a, const void *b);
 void process_ndpi_collected_info(struct ndpi_workflow * workflow, struct ndpi_flow_info *flow);
-u_int32_t ethernet_crc32(const void* data, size_t n_bytes);
 void ndpi_flow_info_free_data(struct ndpi_flow_info *flow);
 void ndpi_flow_info_freer(void *node);
 const char* print_cipher_id(u_int32_t cipher);
-float ndpi_flow_get_byte_count_entropy(const uint32_t byte_count[256], unsigned int num_bytes);
+int parse_proto_name_list(char *str, NDPI_PROTOCOL_BITMASK *bitmask, int inverted_logic);
 
 extern int nDPI_LogLevel;
 
-#ifdef NDPI_ENABLE_DEBUG_MESSAGES
+#if defined(NDPI_ENABLE_DEBUG_MESSAGES) && !defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
 #define LOG(log_level, args...)			\
   {						\
     if(log_level <= nDPI_LogLevel)		\
@@ -414,4 +434,8 @@ extern int nDPI_LogLevel;
 #define LINKTYPE_LINUX_SLL2 276
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
