@@ -8,7 +8,7 @@
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   FuzzedDataProvider fuzzed_data(data, size);
   u_int16_t i, num_iteration, num_learning_values;
-  struct ndpi_jitter_struct s;
+  struct ndpi_jitter_struct *s;
   int rc;
 
   /* Just to have some data */
@@ -18,16 +18,19 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   /* To allow memory allocation failures */
   fuzz_set_alloc_callbacks_and_seed(size);
 
+  s = (struct ndpi_jitter_struct *)ndpi_malloc(sizeof(*s));
+
   num_learning_values = fuzzed_data.ConsumeIntegral<u_int16_t>();
-  rc = ndpi_jitter_init(&s, num_learning_values);
+  rc = ndpi_jitter_init(s, num_learning_values);
 
   if (rc == 0) {
     num_iteration = fuzzed_data.ConsumeIntegral<u_int8_t>();
     for (i = 0; i < num_iteration; i++)
-      ndpi_jitter_add_value(&s, fuzzed_data.ConsumeFloatingPoint<float>());
+      ndpi_jitter_add_value(s, fuzzed_data.ConsumeFloatingPoint<float>());
 
-    ndpi_jitter_free(&s);
+    ndpi_jitter_free(s);
   }
+  ndpi_free(s);
 
   return 0;
 }
