@@ -15,30 +15,20 @@ int nDPI_LogLevel = 0;
 char *_debug_protocols = NULL;
 u_int32_t current_ndpi_memory = 0, max_ndpi_memory = 0;
 u_int8_t enable_protocol_guess = 1, enable_payload_analyzer = 0;
-u_int8_t enable_flow_stats = 0;
+u_int8_t enable_flow_stats = 1;
 u_int8_t human_readeable_string_len = 5;
 u_int8_t max_num_udp_dissected_pkts = 16 /* 8 is enough for most protocols, Signal requires more */, max_num_tcp_dissected_pkts = 80 /* due to telnet */;
-ndpi_init_prefs init_prefs = ndpi_track_flow_payload | ndpi_enable_ja3_plus;
-int enable_malloc_bins = 0;
+ndpi_init_prefs init_prefs = ndpi_track_flow_payload | ndpi_enable_ja3_plus | ndpi_enable_tcp_ack_payload_heuristic;
+int enable_malloc_bins = 1;
 int malloc_size_stats = 0;
-int max_malloc_bins = 0;
+int max_malloc_bins = 14;
 struct ndpi_bin malloc_bins; /* unused */
 
-FILE *bufferToFile(const uint8_t *Data, size_t Size) {
-  FILE *fd;
-  fd = tmpfile();
-  if (fd == NULL) {
-    perror("Error tmpfile");
-    return NULL;
-  }
-  if (fwrite (Data, 1, Size, fd) != Size) {
-    perror("Error fwrite");
-    fclose(fd);
-    return NULL;
-  }
-  rewind(fd);
-  return fd;
-}
+extern void ndpi_report_payload_stats(FILE *out);
+
+#ifdef CRYPT_FORCE_NO_AESNI
+extern int force_no_aesni;
+#endif
 
 int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   pcap_t * pkts;
@@ -76,6 +66,11 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     ndpi_load_malicious_ja3_file(workflow->ndpi_struct, "ja3_fingerprints.csv");
     ndpi_load_malicious_sha1_file(workflow->ndpi_struct, "sha1_fingerprints.csv");
 
+    ndpi_set_detection_preferences(workflow->ndpi_struct, ndpi_pref_enable_tls_block_dissection, 0 /* unused */);
+
+    ndpi_set_monitoring_state(workflow->ndpi_struct, NDPI_PROTOCOL_STUN,
+                              10, NDPI_MONITORING_STUN_SUBCLASSIFIED);
+
     memset(workflow->stats.protocol_counter, 0,
 	   sizeof(workflow->stats.protocol_counter));
     memset(workflow->stats.protocol_counter_bytes, 0,
@@ -83,14 +78,22 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     memset(workflow->stats.protocol_flows, 0,
 	   sizeof(workflow->stats.protocol_flows));
     ndpi_finalize_initialization(workflow->ndpi_struct);
+
+#ifdef CRYPT_FORCE_NO_AESNI
+    force_no_aesni = 1;
+#endif
+
+#ifdef ENABLE_PAYLOAD_ANALYZER
+   enable_payload_analyzer = 1;
+#endif
   }
 
 #ifdef ENABLE_MEM_ALLOC_FAILURES
   /* Don't fail memory allocations until init phase is done */
-  fuzz_set_alloc_seed(Size);
+  fuzz_set_alloc_callbacks_and_seed(Size);
 #endif
 
-  fd = bufferToFile(Data, Size);
+  fd = buffer_to_file(Data, Size);
   if (fd == NULL)
     return 0;
 
@@ -136,6 +139,15 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   for(i = 0; i < workflow->prefs.num_roots; i++)
     ndpi_tdestroy(workflow->ndpi_flows_root[i], ndpi_flow_info_freer);
   ndpi_free(workflow->ndpi_flows_root);
+  /* Free payload analyzer data, without printing */
+  if(enable_payload_analyzer)
+    ndpi_report_payload_stats(stdout);
+
+#ifdef ENABLE_PAYLOAD_ANALYZER
+  ndpi_update_params(SPLT_PARAM_TYPE, "splt_param.txt");
+  ndpi_update_params(BD_PARAM_TYPE, "bd_param.txt");
+  ndpi_update_params(2, ""); /* invalid */
+#endif
 
   return 0;
 }
