@@ -41,8 +41,6 @@
  *
  */
 
-#define _GNU_SOURCE
-
 #include <stdio.h>
 #include <ctype.h>
 // #include <sys/time.h>
@@ -53,6 +51,8 @@
 #include "ndpi_classify.h"
 #include "ndpi_includes.h"
 
+#include "ndpi_replace_printf.h"
+
 /** finds the minimum value between to inputs */
 #ifndef min
 #define min(a,b)				\
@@ -268,10 +268,12 @@ ndpi_merge_splt_arrays (const uint16_t *pkt_len, const pkt_timeval *pkt_time,
                         uint16_t *merged_lens, uint16_t *merged_times)
 {
   int s,r;
-  pkt_timeval ts_start = { 0, 0 }; /* initialize to avoid spurious warnings */
+  pkt_timeval ts_start;
   pkt_timeval tmp, tmp_r;
   pkt_timeval start_m;
 
+  ndpi_timer_clear(&ts_start);
+
   if(r_idx + s_idx == 0) {
     return ;
   } else if(r_idx == 0) {
@@ -583,7 +585,9 @@ ndpi_update_params (classifier_type_codes_t param_type, const char *param_file)
     break;
 
   default:
+#if 0
     printf("error: unknown paramerter type (%d)", param_type);
+#endif
     break;
   }
 }
@@ -677,7 +681,7 @@ ndpi_timeval_to_microseconds(pkt_timeval ts)
 {
   u_int64_t sec = ts.tv_sec;
   u_int64_t usec = ts.tv_usec;
-  return usec + sec * 1000 * 1000;;
+  return usec + sec * 1000 * 1000;
 }
 
 /* **************************************** */
