@@ -38,8 +38,9 @@
    * https://groups.google.com/a/chromium.org/g/proto-quic/c/OAVgFqw2fko/m/jCbjP0AVAAAJ
    * https://groups.google.com/a/chromium.org/g/proto-quic/c/OAVgFqw2fko/m/-NYxlh88AgAJ
    * https://docs.google.com/document/d/1FcpCJGTDEMblAs-Bm5TYuqhHyUqeWpqrItw2vkMFsdY/edit
-   * https://tools.ietf.org/html/draft-ietf-quic-tls-29
-   * https://tools.ietf.org/html/draft-ietf-quic-transport-29
+   * https://www.rfc-editor.org/rfc/rfc9001.txt [Using TLS over QUIC]
+   * https://www.rfc-editor.org/rfc/rfc9000.txt [v1]
+   * https://www.rfc-editor.org/rfc/rfc9369.txt [v2]
    */
 
 extern int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
@@ -50,6 +51,7 @@ extern int http_process_user_agent(struct ndpi_detection_module_struct *ndpi_str
 extern int is_valid_rtp_payload_type(uint8_t type);
 
 /* Versions */
+#define V_2		0x6b3343cf
 #define V_1		0x00000001
 #define V_Q024		0x51303234
 #define V_Q025		0x51303235
@@ -70,6 +72,10 @@ extern int is_valid_rtp_payload_type(uint8_t type);
 
 #define QUIC_MAX_CID_LENGTH  20
 
+static int is_version_forcing_vn(uint32_t version)
+{
+  return (version & 0x0F0F0F0F) == 0x0a0a0a0a; /* Forcing Version Negotiation */
+}
 static int is_version_gquic(uint32_t version)
 {
   return ((version & 0xFFFFFF00) == 0x54303500) /* T05X */ ||
@@ -83,8 +89,8 @@ static int is_version_quic(uint32_t version)
   return version == V_1 ||
     ((version & 0xFFFFFF00) == 0xFF000000) /* IETF Drafts*/ ||
     ((version & 0xFFFFF000) == 0xfaceb000) /* Facebook */ ||
-    ((version & 0x0F0F0F0F) == 0x0a0a0a0a) /* Forcing Version Negotiation */ ||
-    (version == 0x709A50C4);               /* V2 IETF Drafts */
+    is_version_forcing_vn(version) ||
+    (version == V_2);
 }
 static int is_version_valid(uint32_t version)
 {
@@ -108,16 +114,16 @@ static uint8_t get_u8_quic_ver(uint32_t version)
 
   /* "Versions that follow the pattern 0x?a?a?a?a are reserved for use in
      forcing version negotiation to be exercised".
-     It is tricky to return a correct draft version: such number is primarly
-     used to select a proper salt (which depends on the version itself), but
-     we don't have a real version here! Let's hope that we need to handle
-     only latest drafts... */
-  if ((version & 0x0F0F0F0F) == 0x0a0a0a0a)
-    return 29;
+     We can't return a correct draft version because we don't have a real
+     version here! That means that we can't decode any data and we can dissect
+     only the cleartext header.
+     Let's return v1 (any other numbers should be fine, anyway) to only allow
+     the dissection of the (expected) long header */
+  if(is_version_forcing_vn(version))
+    return 34;
 
   /* QUIC Version 2 */
-  /* For the time being use 100 as a number for V2 and let see how v2 drafts evolve */
-  if (version == 0x709A50C4)
+  if (version == V_2)
     return 100;
 
   return 0;
@@ -198,7 +204,48 @@ static int is_version_with_v1_labels(uint32_t version)
 }
 static int is_version_quic_v2(uint32_t version)
 {
-  return version == 0x709A50C4;
+  return version == V_2;
+}
+
+char *ndpi_quic_version2str(char *buf, int buf_len, u_int32_t version) {
+
+  if(buf == NULL || buf_len <= 1)
+    return NULL;
+
+  switch(version) {
+  case V_2: strncpy(buf, "V-2", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_1: strncpy(buf, "V-1", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q024: strncpy(buf, "Q024", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q025: strncpy(buf, "Q025", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q030: strncpy(buf, "Q030", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q033: strncpy(buf, "Q033", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q034: strncpy(buf, "Q034", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q035: strncpy(buf, "Q035", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q037: strncpy(buf, "Q037", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q039: strncpy(buf, "Q039", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q043: strncpy(buf, "Q043", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q046: strncpy(buf, "Q046", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_Q050: strncpy(buf, "Q050", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_T050: strncpy(buf, "T050", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_T051: strncpy(buf, "T051", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_MVFST_22: strncpy(buf, "MVFST-22", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_MVFST_27: strncpy(buf, "MVFST-27", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  case V_MVFST_EXP: strncpy(buf, "MVFST-EXP", buf_len); buf[buf_len - 1] = '\0'; return buf;
+  }
+
+  if(is_version_forcing_vn(version)) {
+    strncpy(buf, "Ver-Negotiation", buf_len);
+    buf[buf_len - 1] = '\0';
+    return buf;
+  }
+  if(((version & 0xFFFFFF00) == 0xFF000000)) {
+    snprintf(buf, buf_len, "Draft-%d", version & 0x000000FF);
+    buf[buf_len - 1] = '\0';
+    return buf;
+  }
+
+  ndpi_snprintf(buf, buf_len, "Unknown (%04X)", version);
+  return buf;
 }
 
 int quic_len(const uint8_t *buf, uint64_t *value)
@@ -246,6 +293,7 @@ static uint16_t gquic_get_u16(const uint8_t *buf, uint32_t version)
 }
 
 
+#ifdef NDPI_ENABLE_DEBUG_MESSAGES
 static char *__gcry_err(gpg_error_t err, char *buf, size_t buflen)
 {
   gpg_strerror_r(err, buf, buflen);
@@ -255,6 +303,7 @@ static char *__gcry_err(gpg_error_t err, char *buf, size_t buflen)
     buf[buflen - 1] = '\0';
   return buf;
 }
+#endif
 
 static uint64_t pntoh64(const void *p)
 {
@@ -874,8 +923,8 @@ static int quic_derive_initial_secrets(struct ndpi_detection_module_struct *ndpi
     0x9a, 0xe6, 0xa4, 0xc8, 0x0c, 0xad, 0xcc, 0xbb, 0x7f, 0x0a
   };
   static const uint8_t handshake_salt_v2_draft_00[20] = {
-    0xa7, 0x07, 0xc2, 0x03, 0xa5, 0x9b, 0x47, 0x18, 0x4a, 0x1d,
-    0x62, 0xca, 0x57, 0x04, 0x06, 0xea, 0x7a, 0xe3, 0xe5, 0xd3
+    0x0d, 0xed, 0xe3, 0xde, 0xf7, 0x00, 0xa6, 0xdb, 0x81, 0x93,
+    0x81, 0xbe, 0x6e, 0x26, 0x9d, 0xcb, 0xf9, 0xbd, 0x2e, 0xd9
   };
   gcry_error_t err;
   uint8_t secret[HASH_SHA2_256_LENGTH];
@@ -1141,7 +1190,6 @@ static const uint8_t *get_reassembled_crypto_data(struct ndpi_detection_module_s
 
 const uint8_t *get_crypto_data(struct ndpi_detection_module_struct *ndpi_struct,
 			       struct ndpi_flow_struct *flow,
-			       uint32_t version,
 			       u_int8_t *clear_payload, uint32_t clear_payload_len,
 			       uint64_t *crypto_data_len)
 {
@@ -1149,6 +1197,7 @@ const uint8_t *get_crypto_data(struct ndpi_detection_module_struct *ndpi_struct,
   uint32_t counter;
   uint8_t first_nonzero_payload_byte, offset_len;
   uint64_t unused, frag_offset, frag_len;
+  u_int32_t version = flow->protos.tls_quic.quic_version;
 
   counter = 0;
   while(counter < clear_payload_len && clear_payload[counter] == 0)
@@ -1322,8 +1371,7 @@ static uint8_t *get_clear_payload(struct ndpi_detection_module_struct *ndpi_stru
 
 void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
 		 struct ndpi_flow_struct *flow,
-		 const u_int8_t *crypto_data, uint32_t crypto_data_len,
-		 uint32_t version)
+		 const u_int8_t *crypto_data, uint32_t crypto_data_len)
 {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
@@ -1335,7 +1383,7 @@ void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
   packet->payload = crypto_data;
   packet->payload_packet_len = crypto_data_len;
 
-  processClientServerHello(ndpi_struct, flow, version);
+  processClientServerHello(ndpi_struct, flow, flow->protos.tls_quic.quic_version);
   flow->protos.tls_quic.hello_processed = 1; /* Allow matching of custom categories */
 
   /* Restore */
@@ -1580,7 +1628,7 @@ static int may_be_initial_pkt(struct ndpi_detection_module_struct *ndpi_struct,
      if the packet includes a token provided by the server in a NEW_TOKEN
      frame on a connection where the server also included the
      grease_quic_bit transport parameter." */
-  if((*version & 0x0F0F0F0F) == 0x0a0a0a0a &&
+  if(is_version_forcing_vn(*version) &&
      !(pub_bit1 == 1 && pub_bit2 == 1)) {
     NDPI_LOG_DBG2(ndpi_struct, "Version 0x%x with first byte 0x%x\n", *version, first_byte);
     return 0;
@@ -1606,8 +1654,10 @@ static int may_be_initial_pkt(struct ndpi_detection_module_struct *ndpi_struct,
 /* ***************************************************************** */
 
 static int eval_extra_processing(struct ndpi_detection_module_struct *ndpi_struct,
-				 struct ndpi_flow_struct *flow, u_int32_t version)
+				 struct ndpi_flow_struct *flow)
 {
+  u_int32_t version = flow->protos.tls_quic.quic_version;
+
   /* For the time being we need extra processing in two cases only:
      1) to detect Snapchat calls, i.e. RTP/RTCP multiplxed with QUIC.
         Two cases:
@@ -1657,6 +1707,9 @@ static int ndpi_search_quic_extra(struct ndpi_detection_module_struct *ndpi_stru
 
   NDPI_LOG_DBG(ndpi_struct, "search QUIC extra func\n");
 
+  if(packet->payload_packet_len == 0)
+    return 1;
+
   if (is_ch_reassembler_pending(flow)) {
     ndpi_search_quic(ndpi_struct, flow);
     if(is_ch_reassembler_pending(flow))
@@ -1701,6 +1754,84 @@ static int ndpi_search_quic_extra(struct ndpi_detection_module_struct *ndpi_stru
   return 0;
 }
 
+static int is_vn(struct ndpi_detection_module_struct *ndpi_struct,
+		 struct ndpi_flow_struct *flow)
+{
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
+  u_int32_t version;
+  u_int8_t first_byte;
+  u_int8_t pub_bit1;
+  u_int8_t dest_conn_id_len, source_conn_id_len;
+
+  /* RFC 8999 6 */
+
+  /* First byte + version (4) + 2 CID lengths (set to 0) + at least one supported version */
+  if(packet->payload_packet_len < 11) {
+    return 0;
+  }
+
+  first_byte = packet->payload[0];
+  pub_bit1 = ((first_byte & 0x80) != 0);
+  if(!pub_bit1) {
+    NDPI_LOG_DBG2(ndpi_struct, "Not a long header\n");
+    return 0;
+  }
+
+  version = ntohl(*((u_int32_t *)&packet->payload[1]));
+  if(version != 0) {
+    NDPI_LOG_DBG2(ndpi_struct, "Invalid version 0x%x\n", version);
+    return 0;
+  }
+
+  /* Check that CIDs lengths are valid: QUIC limits the CID length to 20 */
+  dest_conn_id_len = packet->payload[5];
+  if(5 + 1 + dest_conn_id_len >= packet->payload_packet_len) {
+    NDPI_LOG_DBG2(ndpi_struct, "Invalid Length %d\n", packet->payload_packet_len);
+    return 0;
+  }
+  source_conn_id_len = packet->payload[5 + 1 + dest_conn_id_len];
+  if (dest_conn_id_len > QUIC_MAX_CID_LENGTH ||
+      source_conn_id_len > QUIC_MAX_CID_LENGTH) {
+    NDPI_LOG_DBG2(ndpi_struct, "Invalid CIDs length %u %u",
+		  dest_conn_id_len, source_conn_id_len);
+    return 0;
+  }
+
+  return 1;
+}
+
+static int ndpi_search_quic_extra_vn(struct ndpi_detection_module_struct *ndpi_struct,
+				     struct ndpi_flow_struct *flow)
+{
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
+
+  /* We are elaborating a packet following the Forcing VN, i.e. we are expecting:
+     1) first a VN packet (from the server)
+     2) then a "standard" Initial from the client */
+  /* TODO: could we unify ndpi_search_quic() and ndpi_search_quic_extra_vn() somehow? */
+
+  NDPI_LOG_DBG(ndpi_struct, "search QUIC extra func VN\n");
+
+  if(packet->payload_packet_len == 0)
+    return 1; /* Keep going */
+
+  if(flow->l4.udp.quic_vn_pair == 0) {
+    if(is_vn(ndpi_struct, flow)) {
+      NDPI_LOG_DBG(ndpi_struct, "Valid VN\n");
+      flow->l4.udp.quic_vn_pair = 1;
+      return 1;
+    } else {
+      NDPI_LOG_DBG(ndpi_struct, "Invalid reply to a Force VN. Stop\n");
+      flow->extra_packets_func = NULL;
+      return 0; /* Stop */
+    }
+  } else {
+    flow->extra_packets_func = NULL;
+    ndpi_search_quic(ndpi_struct, flow);
+    return 0;
+  }
+}
+
 static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
 			     struct ndpi_flow_struct *flow)
 {
@@ -1757,6 +1888,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
 
   NDPI_LOG_INFO(ndpi_struct, "found QUIC\n");
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_QUIC, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+  flow->protos.tls_quic.quic_version = version;
 
   /*
    * 3) Skip not supported versions
@@ -1768,6 +1900,20 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
     return;
   }
 
+  /*
+   * 3a) Forcing VN. There is no payload to analyze yet.
+   * Expecteed flow:
+   *  *) C->S: Forcing VN
+   *  *) S->C: VN
+   *  *) C->S: "Standard" Initial with crypto data
+   */
+  if(is_version_forcing_vn(version)) {
+    NDPI_LOG_DBG(ndpi_struct, "Forcing VN\n");
+    flow->max_extra_packets_to_check = 4; /* TODO */
+    flow->extra_packets_func = ndpi_search_quic_extra_vn;
+    return;
+  }
+
   /*
    * 4) Extract the Payload from Initial Packets
    */
@@ -1780,7 +1926,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
   /*
    * 5) Extract Crypto Data from the Payload
    */
-  crypto_data = get_crypto_data(ndpi_struct, flow, version,
+  crypto_data = get_crypto_data(ndpi_struct, flow,
 				clear_payload, clear_payload_len,
 				&crypto_data_len);
 
@@ -1791,7 +1937,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
     if(!is_version_with_tls(version)) {
       process_chlo(ndpi_struct, flow, crypto_data, crypto_data_len);
     } else {
-      process_tls(ndpi_struct, flow, crypto_data, crypto_data_len, version);
+      process_tls(ndpi_struct, flow, crypto_data, crypto_data_len);
     }
   }
   if(is_version_with_encrypted_header(version)) {
@@ -1801,7 +1947,7 @@ static void ndpi_search_quic(struct ndpi_detection_module_struct *ndpi_struct,
   /*
    * 7) We need to process other packets than (the first) ClientHello/CHLO?
    */
-  if(eval_extra_processing(ndpi_struct, flow, version)) {
+  if(eval_extra_processing(ndpi_struct, flow)) {
     flow->max_extra_packets_to_check = 24; /* TODO */
     flow->extra_packets_func = ndpi_search_quic_extra;
   } else if(!crypto_data) {
