@@ -19,9 +19,6 @@
  */
 
 #ifdef __linux__
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <sched.h>
 #endif /* linux */
 
