@@ -1,6 +1,9 @@
-#!/bin/sh
+#!/usr/bin/env bash
+
+set -e
 
 cd "$(dirname "${0}")" || exit 1
+. ./common.sh || exit 1
 
 DEST=../src/lib/inc_generated/ndpi_amazon_aws_match.c.inc
 TMP=/tmp/aws.json
@@ -10,15 +13,15 @@ ORIGIN=https://ip-ranges.amazonaws.com/ip-ranges.json
 
 echo "(1) Downloading file..."
 http_response=$(curl -s -o $TMP -w "%{http_code}" ${ORIGIN})
-if [ $http_response != "200" ]; then
-    echo "Error $http_response: you probably need to update the list url!"
-    exit 1
-fi
+check_http_response "${http_response}"
+is_file_empty "${TMP}"
 
 echo "(2) Processing IP addresses..."
 jq -r '.prefixes | .[].ip_prefix' $TMP > $LIST # TODO: ipv6
+is_file_empty "${LIST}"
 ./ipaddr2list.py $LIST NDPI_PROTOCOL_AMAZON_AWS > $DEST
 rm -f $TMP $LIST
+is_file_empty "${DEST}"
 
 echo "(3) Amazon AWS IPs are available in $DEST"
 exit 0
