@@ -1,6 +1,9 @@
-#!/bin/sh
+#!/usr/bin/env bash
+
+set -e
 
 cd "$(dirname "${0}")" || exit 1
+. ./common.sh || exit 1
 
 DEST=../src/lib/inc_generated/ndpi_cachefly_match.c.inc
 LIST=/tmp/cachefly.list
@@ -9,14 +12,13 @@ ORIGIN='https://cachefly.cachefly.net/ips/cdn.txt'
 
 echo "(1) Downloading file..."
 http_response=$(curl -s -o "${LIST}" -w "%{http_code}" "${ORIGIN}")
-if [ "${http_response}" != "200" ]; then
-    echo "Error ${http_response}: you probably need to update the list url!"
-    exit 1
-fi
+check_http_response "${http_response}"
+is_file_empty "${LIST}"
 
 echo "(2) Processing IP addresses..."
 ./ipaddr2list.py "${LIST}" NDPI_PROTOCOL_CACHEFLY > "${DEST}"
 rm -f "${LIST}"
+is_file_empty "${DEST}"
 
 echo "(3) Cachefly IPs are available in ${DEST}"
 exit 0
