@@ -1,6 +1,9 @@
-#!/bin/sh
+#!/usr/bin/env bash
+
+set -e
 
 cd "$(dirname "${0}")" || exit 1
+. ./common.sh || exit 1
 
 DEST=../src/lib/inc_generated/ndpi_zoom_match.c.inc
 LIST=/tmp/zoom.list
@@ -10,15 +13,14 @@ ORIGIN="https://assets.zoom.us/docs/ipranges/Zoom.txt"
 
 
 echo "(1) Downloading file... ${ORIGIN}"
-http_response=$(curl -s -o $LIST -w "%{http_code}" ${ORIGIN})
-if [ "$http_response" != "200" ]; then
-    echo "Error $http_response: you probably need to update the list url!"
-    exit 1
-fi
+http_response=$(curl -s -o "${LIST}" -w "%{http_code}" "${ORIGIN}")
+check_http_response "${http_response}"
+is_file_empty "${LIST}"
 
 echo "(2) Processing IP addresses..."
-./ipaddr2list.py $LIST NDPI_PROTOCOL_ZOOM > $DEST
-rm -f $LIST
+./ipaddr2list.py "${LIST}" NDPI_PROTOCOL_ZOOM > "${DEST}"
+rm -f "${LIST}"
+is_file_empty "${DEST}"
 
-echo "(3) ZOOM IPs are available in $DEST"
+echo "(3) ZOOM IPs are available in ${DEST}"
 exit 0
